"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Schema_1 = require("./schema/Schema");
const Where_1 = require("./util/Where");
const Utils_1 = require("./util/Utils");
class Delete {
    static delete(conn, pars) {
        let database = pars.database || conn.config.database;
        let table = pars.table;
        if (!table) {
            return Promise.reject(new Error(`pars.table can not be null or empty!`));
        }
        let data = pars.data;
        if (!data) {
            return Promise.reject(new Error(`pars.data can not be null or empty!`));
        }
        return new Promise((resolve, reject) => {
            Schema_1.Schema.getSchema(conn, database).then(schemaModel => {
                let tableSchemaModel = schemaModel.getTableSchemaModel(table);
                if (!tableSchemaModel) {
                    reject(new Error(`Table '${table}' is not exists!`));
                    return;
                }
                let whereList = new Array();
                let whereSQL = ``;
                let primaryKeyList = tableSchemaModel.columns.filter(column => column.primaryKey);
                if (primaryKeyList.length < 1) {
                    reject(new Error(`Table '${table}' has no primary key, you can not call this function. Please try function 'deleteByWhere'!`));
                    return;
                }
                for (let column of primaryKeyList) {
                    if (Reflect.has(data, column.columnName)) {
                        whereSQL += ` ${column.columnName}=? and`;
                        whereList.push(Reflect.get(data, column.columnName));
                    }
                    else {
                        reject(new Error(`Key ${column.columnName} is not provided!`));
                        return;
                    }
                }
                whereSQL = ` where ` + whereSQL.replace(/and$/, "");
                let tableName = Utils_1.Utils.getDbObjectName(database, table);
                let sql = `delete from ${tableName} ${whereSQL}`;
                conn.query(sql, whereList, (err, result) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve();
                    }
                });
            });
        });
    }
    static deleteByWhere(conn, pars) {
        let database = pars.database || conn.config.database;
        let where = pars.where;
        let table = pars.table;
        if (!table) {
            return Promise.reject(new Error(`pars.table can not be null or empty!`));
        }
        return new Promise((resolve, reject) => {
            Schema_1.Schema.getSchema(conn, database).then(schemaModel => {
                let tableSchemaModel = schemaModel.getTableSchemaModel(table);
                if (!tableSchemaModel) {
                    reject(new Error(`Table '${table}' is not exists!`));
                    return;
                }
                let { whereSQL, whereList } = Where_1.Where.getWhereSQL(where, tableSchemaModel);
                let tableName = Utils_1.Utils.getDbObjectName(database, table);
                let sql = `delete from ${tableName} ${whereSQL}`;
                conn.query(sql, whereList, (err, result) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve();
                    }
                });
            });
        });
    }
}
exports.Delete = Delete;
//# sourceMappingURL=Delete.js.map