"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const SaveType_1 = require("./model/SaveType");
const Insert_1 = require("./Insert");
const Update_1 = require("./Update");
const Delete_1 = require("./Delete");
const Replace_1 = require("./Replace");
const Transaction_1 = require("./Transaction");
class Save {
    static save(conn, pars) {
        switch (pars.saveType) {
            case SaveType_1.SaveType.insert: {
                return Insert_1.Insert.insert(conn, {
                    data: pars.data,
                    database: pars.database,
                    table: pars.table
                });
            }
            case SaveType_1.SaveType.update: {
                return Update_1.Update.update(conn, {
                    data: pars.data,
                    database: pars.database,
                    table: pars.table
                });
            }
            case SaveType_1.SaveType.delete: {
                return Delete_1.Delete.delete(conn, {
                    data: pars.data,
                    database: pars.database,
                    table: pars.table
                });
            }
            case SaveType_1.SaveType.replace: {
                return Replace_1.Replace.replace(conn, {
                    data: pars.data,
                    database: pars.database,
                    table: pars.table
                });
            }
        }
    }
    static async saves(conn, list) {
        let promiseList = new Array();
        list.map(h => {
            promiseList.push(Save.save(conn, {
                data: h.data,
                database: h.database,
                table: h.table,
                saveType: h.saveType
            }));
        });
        return Promise.all(promiseList);
    }
    static savesWithTran(conn, list) {
        return new Promise((resolve, reject) => {
            (async function () {
                try {
                    await Transaction_1.Transaction.begin(conn);
                    await Save.saves(conn, list);
                    await Transaction_1.Transaction.commit(conn);
                    resolve();
                }
                catch (err) {
                    await Transaction_1.Transaction.rollback(conn);
                    reject(err);
                }
            })();
        });
    }
    static async savesSeq(conn, list) {
        for (let item of list) {
            await Save.save(conn, item);
        }
    }
    static async savesSeqWithTran(conn, list) {
        try {
            await Transaction_1.Transaction.begin(conn);
            for (let item of list) {
                await Save.save(conn, item);
            }
            await Transaction_1.Transaction.commit(conn);
            return Promise.resolve();
        }
        catch (err) {
            await Transaction_1.Transaction.rollback(conn);
            return Promise.reject(err);
        }
    }
}
exports.Save = Save;
//# sourceMappingURL=Save.js.map